/* Copyright (c) 2006,2007,2008,2009 Polybus Systems Corp., Westford, MA, USA  */

/* All rights reserved. */

/* Redistribution and use in source and binary forms, with or without modification, */
/* are permitted provided that the following conditions are met: */

/* * Redistributions of source code must retain the above copyright notice, this */
/* list of conditions and the following disclaimer. */
/* * Redistributions in binary form must reproduce the above copyright notice, */
/* this list of conditions and the following disclaimer in the documentation */
/* and/or other materials provided with the distribution. */
/* * Neither the name of the Polybus Systems Corp nor the names of its contributors */
/* may be used to endorse or promote products derived from this software */
/* without specific prior written permission. */

/* THIS SOFTWARE IS PROVIDED BY POLYBUS AND CONTRIBUTORS ''AS IS'' AND ANY */
/* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE */
/* DISCLAIMED. IN NO EVENT SHALL POLYBUS OR CONTRIBUTORS BE LIABLE FOR ANY */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES */
/* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON */
/* ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT */
/* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS */
/* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */

#include "sys_basher.h"

static FILE *transcript=0;
static Boolean verbose=0;
static long file_num=-1;

static char report[1024];

void openTranscript(char *report_name)
{
  sprintf(report,"%s.log",report_name);
  transcript = fopen(report,"w");
  if(transcript)
    {
      file_num = fileno(transcript);
    }
}

void closeTranscript(void)
{
  if(transcript) fclose(transcript);
}

void printTranscript(char *string,Boolean print_always)
{
  if(transcript)
    {
      fprintf(transcript,"%s",string);
    }
  if(verbose || print_always) printf("%s",string);
}

void flushTranscript(void)
{
  fclose(transcript);
  sync();
  transcript = fopen(report,"a");
}

void setVerbose(void)
{
  verbose = TRUE;
}

void memSize(char *string,SInt64 size)
{
  char xstring[256];
  SInt64 scaled;
  
  if(size >= 0x40000000LL)
    {
      scaled = size/0x40000000LL;
      sprintf(xstring,"%lld GBytes",scaled);
    }
  else if(size >= 0x100000LL)
    {
      scaled = size/0x100000LL;
      sprintf(xstring,"%lld MBytes",scaled);
    }
  else
    {
      scaled = size/1024LL;
      sprintf(xstring,"%lld KBytes",scaled);
    }
  rightJustifyString(xstring,string,10);
}

void bandWidth(char *prefix,SInt64 length,double rusecs,double wusecs)
{
  double len,rd,wr;
  char src[256],string[512],rbw[256],wbw[256];
  len = length;
  rd = len/rusecs;
  wr = len/wusecs;
  if(rd >= 1024.0)
    {
      sprintf(src,"%2.2f",rd/1024.0);
      rightJustifyString(src,string,4);
      sprintf(rbw,"%s GB/S",string);
    }
  else if(rd < 1.0)
    {
      sprintf(src,"%2.2f",rd * 1024.0);
      rightJustifyString(src,string,4);
      sprintf(rbw,"%s KB/S",string);
    }
  else
    {
      sprintf(src,"%2.2f",rd);
      rightJustifyString(src,string,4);
      sprintf(rbw,"%s MB/S",string);
    }
  if(wr >= 1024.0)
    {
      sprintf(src,"%2.2f",wr/1024.0);
      rightJustifyString(src,string,4);
      sprintf(wbw,"%s GB/S",string);
    }
  else if(wr < 1.0)
    {
      sprintf(src,"%2.2f",wr * 1024.0);
      rightJustifyString(src,string,4);
      sprintf(wbw,"%s KB/S",string);
    }
  else
    {
      sprintf(src,"%2.2f",wr);
      rightJustifyString(src,string,4);
      sprintf(wbw,"%s MB/S",string);
    }

  sprintf(string,"%s, Read Bandwidth %s, Write Bandwidth %s\n",prefix,
	 rbw,wbw);
  printTranscript(string,FALSE);
}

// Return the time in uSeconds
double getTime(void)
{
  time_def t;
  double usecs,secs;
  gettimeofday((void *)&t,(struct timezone *)NULL);
  secs = t.sec;
  secs *= 0x100000LL;
  usecs = t.usecs;
  usecs += secs;
  return(usecs);
  
}

void delay(SInt32 seconds)
{
  double endtime,mytime;
  mytime = getTime();
  endtime = seconds;
  endtime *= 1000000.0;
  endtime += mytime;
  while(mytime < endtime)
    {
      mytime = getTime();
    }
}
   
void printTime(void)
{
  time_t   now;
  struct   tm    *date;
  char string[512],estring[512];

  now = time(NULL);
  date = localtime( &now );
  strftime(string,80,"%B %d %Y %T",date);
  sprintf(estring,"\n%s\n\n",string);
  printTranscript(estring,TRUE);

}

long getPathName(long num,long size,long indx,file_info *finfo,char *string)
{
  char dstring[1024];
  if(getDirName(num,indx,finfo,dstring))
    {
      sprintf(string,"%s/tf_%ld",dstring,size);
      return(TRUE);
    }
  return(FALSE);
}

long getDirName(long num,long indx,file_info *finfo,char *string)
{
  if(num >= 0)
    {
      if((finfo->num_paths == 0) && (indx == 0))
	{
	  sprintf(string,"./disk_test/test_%ld",num);
	  return(TRUE);
	}
      else if(indx < finfo->num_paths)
	{
	  sprintf(string,"%s/disk_test/test_%ld",finfo->path[indx],num);
	  return(TRUE);
	}
    }
  else 
    {
      if((finfo->num_paths == 0) && (indx == 0))
	{
	  sprintf(string,"./disk_test");
	  return(TRUE);
	}
      else if(indx < finfo->num_paths)
	{
	  sprintf(string,"%s/disk_test",finfo->path[indx]);
	  return(TRUE);
	}
      
    }
  return(FALSE);
}

void pathCopy(char *des,char *src,long limit)
{
  char *sindx,*dindx,myChar,lastChar=0,i=0;
  sindx = src;
  dindx = des;
  myChar = *sindx++;
  while((myChar != 0) && (i < limit))
    {
      *dindx++ = myChar;
      lastChar = myChar;
      myChar = *sindx++;
      i++;
    }
  if(lastChar == '/')
    {
      dindx--;
    }
  *dindx = 0;
}

long strToInt(char *srcString)
{
  long acc=0;
  char myChar,*srcIndx,digit;
  char done=0,neg=0;
	
  srcIndx = srcString;
	
  while(!done)
    {
      myChar = *srcIndx++;
      done = ((myChar >= '0') && (myChar <= '9'));
      if(myChar == '-') neg = 1;
      if(myChar == 0) return(FALSE);
    }
  done = 0;
  while(!done)
    {
      if((myChar >= '0') && (myChar <= '9'))
	{
	  digit = myChar - '0';
	  acc *= 10L;
	  acc += (long)digit;
	}
      else
	{
	  done = 1;
	}
      myChar = *srcIndx++;
    }
  if(neg) acc = -acc;
  return(acc);
}


void rightJustifyString(char *src,char *des,long width)
{
  char *sindx,*dindx,myChar;
  long length,i;
  length = strlen(src);
  sindx = src;
  dindx = des;
  for(i=length;i<width;i++)
    {
      *dindx++ = ' ';
    }
  myChar = *sindx++;
  while(myChar)
    {
      *dindx++ = myChar;
      myChar = *sindx++;
    }
  *dindx++ = 0;
}


void leftJustifyString(char *src,char *des,long width)
{
  char *sindx,*dindx,myChar;
  long length,i;
  length = strlen(src);
  sindx = src;
  dindx = des;
  myChar = *sindx++;
  while(myChar)
    {
      *dindx++ = myChar;
      myChar = *sindx++;
    }
  for(i=length;i<width;i++)
    {
      *dindx++ = ' ';
    }
  *dindx++ = 0;
}

void scale(char *string,char *postfix,double value)
{
  double scaled;
  if(value >= 1024.0)
    {
      scaled = value/1024.0;
      sprintf(string,"%2.2fG%s",scaled,postfix);
    }
  else if(value < 1.0)
    {
      scaled = value * 1024.0;
      sprintf(string,"%2.2fK%s",scaled,postfix);
    }
  else
    {
      sprintf(string,"%2.2fM%s",value,postfix);
    }
}

void iscale(char *string,char *postfix,double value)
{
  double scaled,div1,div2;
  
  div1 = 0x40000000LL;
  div2 = 0x100000LL;
  if(value >= div1)
    {
      scaled = value/div1;
      sprintf(string,"%0.0fG%s",scaled,postfix);
    }
  else if(value >= div2)
    {
      scaled = value/div2;
      sprintf(string,"%0.0fM%s",scaled,postfix);
    }
  else
    {
      scaled = value/1024.0;
      sprintf(string,"%0.0fK%s",scaled,postfix);
    }
}

void mprint(long width,char *prefix,SInt64 value)
{
  char string[256],estring[256];
  mscale(width,string,prefix,"Bytes",value);
  sprintf(estring,"%s\n",string);
  printTranscript(estring,TRUE);
}

void mscale(long width,char *string,char *prefix,char *postfix,SInt64 ivalue)
{
  double scaled,value,div1,div2;
  char ystring[256],xstring[256];
  sprintf(xstring,"%s:",prefix);
  leftJustifyString(xstring,ystring,width);
  value = ivalue;

  div1 = 0x40000000LL;
  div2 = 0x100000LL;
  if(value >= div1)
    {
      scaled = value/div1;
      sprintf(string,"%s%1.3f G%s",ystring,scaled,postfix);
    }
  else if(value >= div2)
    {
      scaled = value/div2;
      sprintf(string,"%s%1.3f M%s",ystring,scaled,postfix);
    }
  else
    {
      scaled = value/1024.0;
      sprintf(string,"%s%1.3f K%s",ystring,scaled,postfix);
    }
}

void displayLine(char *prefix,char *unit,double values[],long count)
{
  char string[1024],xstring[64],ystring[64];
  long i;

  leftJustifyString(prefix,string,16);
  for(i=0;i<count;i++)
    {
      scale(xstring,unit,values[i]);
      leftJustifyString(xstring,ystring,13);
      sprintf(string,"%s%s",string,ystring);
    }
  sprintf(string,"%s\n",string);
  printTranscript(string,FALSE);
  
}

void displayHexLine(char *prefix,char *unit,double values[],long count)
{
  char string[1024],xstring[64];
  long i;
  SInt64 *hex;

  leftJustifyString(prefix,string,16);
  hex = (SInt64*)values;
  for(i=0;i<count;i++)
    {
      sprintf(xstring,"0x%16.16llXLL,\n",hex[i]);
      printTranscript(xstring,FALSE);
    }
  sprintf(string,"%s\n",string);
  printTranscript(string,FALSE);
  
}

void displayHexFloat(char *prefix,char *unit,float values[],long count)
{
  char string[1024],xstring[64];
  long i;
  unsigned long *hex;

  leftJustifyString(prefix,string,16);
  hex = (unsigned long *)values;
  for(i=0;i<count;i++)
    {
      sprintf(xstring,"0x%8.8lXL,\n",hex[i]);
      printTranscript(xstring,FALSE);
    }
  sprintf(string,"%s\n",string);
  printTranscript(string,FALSE);
  
}

long displayPassFail(char *prefix,double values[],double exp[],long count)
{
  char string[1024];
  static char *pass = "PASS         ";
  static char *fail = "FAIL         ";
  long i,errors=0;

  leftJustifyString(prefix,string,16);
  for(i=0;i<count;i++)
    {
      if(values[i] == exp[i])
	{
	  sprintf(string,"%s%s",string,pass);
	}
      else
	{
	  sprintf(string,"%s%s",string,fail);
	  errors++;
	}
    }
  sprintf(string,"%s\n",string);
  printTranscript(string,FALSE);
  return(errors);
  
}

long displayFloatPassFail(char *prefix,float values[],float exp[],long count)
{
  char string[1024];
  static char *pass = "PASS         ";
  static char *fail = "FAIL         ";
  long i,errors=0;

  leftJustifyString(prefix,string,16);
  for(i=0;i<count;i++)
    {
      if(values[i] == exp[i])
	{
	  sprintf(string,"%s%s",string,pass);
	}
      else
	{
	  sprintf(string,"%s%s",string,fail);
	  errors++;
	}
    }
  sprintf(string,"%s\n",string);
  printTranscript(string,FALSE);

  return(errors);
  
}

void displayTitles(char *prefix,char *values[],long count)
{
  char string[4096],ystring[64];
  long i;

  leftJustifyString(prefix,string,16);
  for(i=0;i<count;i++)
    {
      leftJustifyString(values[i],ystring,13);
      sprintf(string,"%s%s",string,ystring);
    }
  sprintf(string,"%s\n",string);
  printTranscript(string,FALSE);
  
}

void printTestName(char *name,SInt64 length)
{
  char string[256],nstring[256],xstring[512];
  double len=length;

  iscale(string,"B",len);
  leftJustifyString(name,nstring,16);
  sprintf(xstring,"%sArray Size: %s\n",nstring,string);
  printTranscript(xstring,TRUE);
  
}

void displayStatusLine(char *prefix,char *unit,double values[],long count,long status)
{
  char string[1024],xstring[1024],ystring[1024];
  long i;

  leftJustifyString(prefix,string,16);
  for(i=0;i<count;i++)
    {
      scale(xstring,unit,values[i]);
      leftJustifyString(xstring,ystring,13);
      sprintf(string,"%s%s",string,ystring);
    }
  if(status)
    {
      sprintf(xstring,"%sFAIL\n",string);
      printTranscript(xstring,FALSE);
    }
  else
    {
      sprintf(xstring,"%sPASS\n",string);
      printTranscript(xstring,FALSE);
    }
  
}

void commaFormat(char *string,SInt64 value)
{
  SInt64 acc,mod,scaler;
  long firstTime=1;
  acc = value;
  string[0] = 0;
  scaler =  1000000000000LL;
  if(value == 0)
    {
      sprintf(string,"0");
      return;
    }
  while(scaler != 0)
    {
      mod = acc/scaler;
      if(firstTime)
	{
	  if(mod != 0)
	    {
	      firstTime = 0;
	      sprintf(string,"%3.3lld",mod);
	    }
	}
      else
	{
	  sprintf(string,"%s,%3.3lld",string,mod);
	}
      mod *= scaler;
      acc -= mod;
      scaler /= 1000LL;

    }
}

void cpuRest(double ttime,double dutyscale)
{
  double testtime;
  unsigned long usecs;

  if(dutyscale == 0.0) return;
  testtime = ttime * dutyscale;
  usecs = testtime;
  if(usecs == 0L) usecs = 1L;
  usleep(usecs);
  
}

void cpyUpCase(char *des, char *src)
{
  strncpy(des,src,32);
  des[31] = 0;
  upCaseString(des);
}
void downCaseString(char *string)
{
  static char xlate[256];
  long i,value;
  static char firstTime=1;
	
  if(firstTime)
    {
      for(i=0;i<256;i++) xlate[i] = i;
      value = 'a';
      for(i='A';i<='Z';i++)
	{
	  xlate[i] = value;
	  value++;
	}
      firstTime = 0;
    }
  translateString(xlate,string);
}

void translateString(char xlateTable[],char *string)
{
  long length,i;
  unsigned char *desIndx;
  unsigned long oldChar;
  desIndx = (unsigned char *)string;
  length = strlen(string);
  for(i=0;i<length;i++)
    {
      oldChar = *desIndx;
      *desIndx++ = xlateTable[oldChar];
    }
  *desIndx = 0;
}


void upCaseString(char *string)
{
  static char xlate[256];
  long i,value;
  static Boolean firstTime=TRUE;
	
  if(firstTime)
    {
      for(i=0;i<256;i++) xlate[i] = i;
      value = 'A';
      for(i='a';i<='z';i++)
	{
	  xlate[i] = value;
	  value++;
	}
      firstTime = FALSE;
    }
  translateString(xlate,string);
}


Boolean findSubString(char *src,char *match)
{
  char *indx,*mindx,sChar,mChar,*xindx;

  indx = src;
  mindx = match;

  sChar = *indx++;
  mChar = *mindx++;

  while(sChar)
    {
      if(sChar == mChar)
	{
	  xindx = indx;
	  sChar = *xindx++;
	  mChar = *mindx++;
	  while((sChar != 0) && (mChar != 0) && (sChar == mChar))
	    {
	      sChar = *xindx++;
	      mChar = *mindx++;
	    }
	  if(mChar == 0) return(TRUE);
	  if(sChar == 0) return(FALSE);
	  mindx = match;
	  mChar = *mindx++;
	  
	}
      sChar = *indx++;
    }
  return(FALSE);

}

void printSensors(FILE *fp,long num_temps,char temp_names[64][64],double value[],char *prefix)
{
  long i,type=0,lasttype=0;
  char string[1024],label[1024],vstring[1024];
  static char *unit[3]={"C","V"," RPM"};
  if(num_temps == 0) return;
  if(prefix)
    {
      sprintf(string,"\n%s Temperatures, Voltages and Fan Speeds:\n",prefix);
      if(fp)fprintf(fp,"%s",string);
      printTranscript(string,TRUE);
    }
  else
    {
      sprintf(string,"\nTemperatures, Voltages and Fan Speeds:\n");
      if(fp)fprintf(fp,"%s",string);
      printTranscript(string,TRUE);
    }
  for(i=0;i<num_temps;i++)
    {
      cpyUpCase(label,temp_names[i]);
      if(findSubString(label,"VOLT"))
	{
	  type = 1;
	  sprintf(vstring,"%2.2f",value[i]);
	}
      else if(findSubString(label,"FAN"))
	{
	  type = 2;
	  sprintf(vstring,"%2.0f",value[i]);
	}
      else
	{
	  type = 0;
	  sprintf(vstring,"%2.0f",value[i]);
	}
      sprintf(label,"%s:",temp_names[i]);
      pad2length(label,20L);
      if(type != lasttype)
	{
	  sprintf(string,"\n%s%s%s\n",label,vstring,unit[type]);
	}
      else
	{
	  sprintf(string,"%s%s%s\n",label,vstring,unit[type]);
	}
      lasttype = type;
      if(fp)fprintf(fp,"%s",string);
      printTranscript(string,TRUE);
    }
  if(fp)fprintf(fp,"\n");
  printTranscript("\n",TRUE);
}

void printMinMaxSensors(FILE *fp,FILE *csv,long num_temps,char temp_names[64][64],double min[],double max[])
{
  long i;
  char string[1024],label[1024];
  if(num_temps == 0) return;

  sprintf(string,"\nTemp Sensor\tMin C\tMax C\n");
  if(fp)fprintf(fp,"%s",string);
  printTranscript(string,TRUE);

  if(csv)fprintf(csv,"\nTemp Sensor,Min C,Max C\n");

  for(i=0;i<num_temps;i++)
    {
      cpyUpCase(label,temp_names[i]);
      if(!((findSubString(label,"VOLT")) || (findSubString(label,"FAN"))))
	{
	  sprintf(string,"%s:\t%2.0f\t%2.0f\n",temp_names[i],min[i],max[i]);
	  if(fp)fprintf(fp,"%s",string);
	  printTranscript(string,TRUE);
	  if(csv)fprintf(csv,"%s,%2.0f,%2.0f\n",temp_names[i],min[i],max[i]);
	}
    }

  sprintf(string,"\nVoltage Sensor\tMin V\tMax V\n");
  if(fp)fprintf(fp,"%s",string);
  printTranscript(string,TRUE);

  if(csv)fprintf(csv,"\nVoltage Sensor,Min V,Max V\n");

  for(i=0;i<num_temps;i++)
    {
      cpyUpCase(label,temp_names[i]);
      if(findSubString(label,"VOLT"))
	{
	  sprintf(string,"%s:\t%2.2f\t%2.2f\n",temp_names[i],min[i],max[i]);
	  if(fp)fprintf(fp,"%s",string);
	  printTranscript(string,TRUE);
	  if(csv)fprintf(csv,"%s,%2.2f,%2.2f\n",temp_names[i],min[i],max[i]);
	}
    }

  sprintf(string,"\nFan Sensor\tMin RPM\tMax RPM\n");
  if(fp)fprintf(fp,"%s",string);
  printTranscript(string,TRUE);

  if(csv)fprintf(csv,"\nFan Sensor,Min RPM,Max RPM\n");

  for(i=0;i<num_temps;i++)
    {
      cpyUpCase(label,temp_names[i]);
      if(findSubString(label,"FAN"))
	{
	  sprintf(string,"%s:\t%2.0f\t%2.0f\n",temp_names[i],min[i],max[i]);
	  if(fp)fprintf(fp,"%s",string);
	  printTranscript(string,TRUE);
	  if(csv)fprintf(csv,"%s,%2.0f,%2.0f\n",temp_names[i],min[i],max[i]);
	}
    }

  if(fp)fprintf(fp,"\n");
  if(csv)fprintf(csv,"\n");
  printTranscript("\n",TRUE);
}

Boolean getSysInfo(char **buffer,char *param,SInt64 *value)
{
  char *indx=*buffer,myChar,*sindx,*nindx,num_string[256];
  Boolean found=FALSE,colon=FALSE;
  
  myChar = *indx++;
  sindx = param;
  *sindx = 0;
  nindx = num_string;
  *nindx = 0;
  while(myChar)
    {
      if(((myChar >= 'a') && (myChar <= 'z')) ||
	 ((myChar >= 'A') && (myChar <= 'Z')))
	{
	  if(colon)
	    {
	      indx--;
	      *buffer = indx;
	      param[0] = 0;
	      return(TRUE);
	    }
	  *sindx++ = myChar;
	}
      else if(myChar == ':')
	{
	  *sindx++ = 0;
	  colon = TRUE;
	}
      else if(((myChar >= '0') && (myChar <= '9')) && colon)
	{
	  *nindx++ = myChar;
	  found = TRUE;
	}
      else if(found)
	{
	  *indx++ = 0;
	  *value = decimalToInt(num_string);
	  *buffer = indx;
	  return(TRUE);
	}
      else if(!colon)
	{
	  sindx = param;
	}
      myChar = *indx++;
	
    }
  *buffer = indx;
  return(FALSE);
  
}


SInt64 decimalToInt(char *srcString)
{
  SInt64 acc=0;
  char myChar,*srcIndx,digit;
  Boolean done=FALSE,neg=FALSE;
	
  srcIndx = srcString;
	
  while(!done)
    {
      myChar = *srcIndx++;
      done = ((myChar >= '0') && (myChar <= '9'));
      if(myChar == '-') neg = TRUE;
      if(myChar == 0) return(0);
    }
  done = FALSE;
  while(!done)
    {
      if((myChar >= '0') && (myChar <= '9'))
	{
	  digit = myChar - '0';
	  acc *= 10L;
	  acc += (SInt64)digit;
	}
      else
	{
	  done = TRUE;
	}
      myChar = *srcIndx++;
    }
  if(neg) acc = -acc;
  return(acc);
}

Boolean testEnb(long test,UInt64 select)
{
  switch(test)
    {
    case ADDR_EQ0:
    case ADDR_EQ10:
    case ADDR_EQ11:
    case ADDR_EQ12:
    case ADDR_EQ13:
    case ADDR_EQ14:
    case ADDR_EQ15:
    case ADDR_EQ16:
    case ADDR_EQ17:
    case ADDR_EQ18:
    case ADDR_EQ19:
    case ADDR_EQ1:
    case ADDR_EQ20:
    case ADDR_EQ21:
    case ADDR_EQ22:
    case ADDR_EQ23:
    case ADDR_EQ24:
    case ADDR_EQ25:
    case ADDR_EQ26:
    case ADDR_EQ27:
    case ADDR_EQ28:
    case ADDR_EQ29:
    case ADDR_EQ2:
    case ADDR_EQ30:
    case ADDR_EQ31:
    case ADDR_EQ3:
    case ADDR_EQ4:
    case ADDR_EQ5:
    case ADDR_EQ6:
    case ADDR_EQ7:
    case ADDR_EQ8:
    case ADDR_EQ9:
      return((select == 0L) || ((select & ADDR_EQ_ONLY) != 0));
      break;
    case BANK_TST2:
    case BANK_TST3:
    case BANK_TST4:
    case BANK_TST5:
    case BANK_TST6:
    case BANK_TST7:
    case BANK_TST8:
    case BANK_TST9:
    case BANK_TST10:
    case BANK_TST11:
    case BANK_TST12:
    case BANK_TST13:
    case BANK_TST14:
    case BANK_TST15:
      return((select == 0L) || ((select & BANK_ONLY) != 0));
      break;
    case FIXED_MEM0:
    case FIXED_MEM1:
    case FIXED_MEM2:
    case FIXED_MEM3:
    case FIXED_MEM4:
    case FIXED_MEM5:
    case FIXED_MEM6:
    case FIXED_MEM7:
    case FIXED_MEM8:
    case FIXED_MEM9:
    case FIXED_MEM10:
    case FIXED_MEM11:
    case FIXED_MEM12:
    case FIXED_MEM13:
    case FIXED_MEM14:
    case FIXED_MEM15:
      return((select == 0L) || ((select & PATTERN_ONLY) != 0));
      break;
    case LFSR_TST0:
    case LFSR_TST1:
    case LFSR_TST2:
    case LFSR_TST3:
    case LFSR_TST4:
    case LFSR_TST5:
    case LFSR_TST6:
    case LFSR_TST7:
      return((select == 0L) || ((select & RANDOM_ONLY) != 0));
      break;
    case WALK_TST0:
    case WALK_TST1:
    case WALK_TST2:
    case WALK_TST3:
    case WALK_TST4:
    case WALK_TST5:
    case WALK_TST6:
    case WALK_TST7:
      return((select == 0L) || ((select & WALKING_ONLY) != 0));
      break;
    case REV_ADDR0:
    case REV_ADDR1:
      return((select == 0L) || ((select & REV_ADDR_ONLY) != 0));
      break;
    case MEM_BAND: return((select == 0L) || ((select & BAND_ONLY) != 0L));
    case FP_TST: return((select == 0L) || ((select & FP_ONLY) != 0L));
    case INT_TST: return((select == 0L) || ((select & INT_ONLY) != 0L));
    case DISK_TST: return((select == 0L) || ((select & DISK_ONLY) != 0L));
    case SWAP_TST: return((select & SWAP_ONLY) != 0L);
    default: return(0L);
    }
    
}



SInt32 displayLongPassFail(char *prefix,SInt32 values[],SInt32 exp[],SInt32 count)
{
  char string[1024];
  static char *pass = "PASS         ";
  static char *fail = "FAIL         ";
  long i,errors=0;

  leftJustifyString(prefix,string,16);
  for(i=0;i<count;i++)
    {
      if(values[i] == exp[i])
	{
	  sprintf(string,"%s%s",string,pass);
	}
      else
	{
	  sprintf(string,"%s%s",string,fail);
	  errors++;
	}
    }
  sprintf(string,"%s\n",string);
  printTranscript(string,FALSE);
  return(errors);
  
}

long displaySInt64PassFail(char *prefix,SInt64 values[],SInt64 exp[],long count)
{
  char string[1024];
  static char *pass = "PASS         ";
  static char *fail = "FAIL         ";
  long i,errors=0;

  leftJustifyString(prefix,string,16);
  for(i=0;i<count;i++)
    {
      if(values[i] == exp[i])
	{
	  sprintf(string,"%s%s",string,pass);
	}
      else
	{
	  sprintf(string,"%s%s",string,fail);
	  errors++;
	}
    }
  sprintf(string,"%s\n",string);
  printTranscript(string,FALSE);
  return(errors);
  
}

void updateBandwidth(double bandwidth,double xtime,double ops,result *rslt)
{
  if(rslt->runs == 0)
    {
      rslt->total_time = xtime;
      rslt->total = ops;
      rslt->min = bandwidth;
      rslt->max = bandwidth;
    }
  else
    {
      rslt->total_time += xtime;
      rslt->total += ops;
      if(bandwidth < rslt->min)
	{
	  rslt->min = bandwidth;
	}

      if(bandwidth > rslt->max)
	{
	  rslt->max = bandwidth;
	}
    }
}


void chkDouble(long firsttime,double exp,double got,result *rslt)
{
  if(firsttime) return;

  rslt->runs = rslt->runs + 1;

  if(exp != got)
    {
      rslt->errors = rslt->errors + 1;
    }
}

void chkFloat(long firsttime,float exp,float got,result *rslt)
{
  if(firsttime) return;

  rslt->runs = rslt->runs + 1;

  if(exp != got)
    {
      rslt->errors = rslt->errors + 1;
    }
}


void chkSInt16(long firsttime,SInt16 exp,SInt16 got,result *rslt)
{
  if(firsttime) return;

  rslt->runs = rslt->runs + 1;

  if(exp != got)
    {
      rslt->errors = rslt->errors + 1;
    }
}

void chkSInt32(long firsttime,SInt32 exp,SInt32 got,result *rslt)
{
  if(firsttime) return;

  rslt->runs = rslt->runs + 1;

  if(exp != got)
    {
      rslt->errors = rslt->errors + 1;
    }
}

void chkSInt64(long firsttime,SInt64 exp,SInt64 got,result *rslt)
{
  if(firsttime) return;

  rslt->runs = rslt->runs + 1;

  if(exp != got)
    {
      rslt->errors = rslt->errors + 1;
    }
}


void chkUInt16(long firsttime,UInt16 exp,UInt16 got,result *rslt)
{
  if(firsttime) return;

  rslt->runs = rslt->runs + 1;

  if(exp != got)
    {
      rslt->errors = rslt->errors + 1;
    }
}

void chkUInt32(long firsttime,UInt32 exp,UInt32 got,result *rslt)
{
  if(firsttime) return;

  rslt->runs = rslt->runs + 1;

  if(exp != got)
    {
      rslt->errors = rslt->errors + 1;
    }
}

void chkUInt64(long firsttime,UInt64 exp,UInt64 got,result *rslt)
{
  if(firsttime) return;

  rslt->runs = rslt->runs + 1;

  if(exp != got)
    {
      rslt->errors = rslt->errors + 1;
    }
}

void averageAllResults(SInt32 from,SInt32 to,result_summary results[MAX_THREADS],result_summary *final_results,run_summary *ran)
{
  SInt32 i,j,k,l,m,limit,start;

  if(to < MAX_THREADS)
    {
      limit = to;
    }
  else
    {
      limit = MAX_THREADS;
    }

  if(from < MAX_THREADS)
    {
      start = from;
    }
  else
    {
      start = MAX_THREADS;
    }

  ran->addreq = 0;
  ran->bank = 0;
  ran->dbl = 0;
  ran->disk = 0;
  ran->fixed = 0;
  ran->int16 = 0;
  ran->int32 = 0;
  ran->int64 = 0;
  ran->lfsr = 0;
  ran->mem_bw = 0;
  ran->rev_addr = 0;
  ran->sgl = 0;
  ran->swap = 0;
  ran->walk = 0;

  for(i=start;i<limit;i++)
    {
      for(j=0;j<NUM_FIXED;j++)
	{
	  ran->fixed |= averageResults(&results[i].fixed[j],&final_results->fixed[j]);
	}
      for(j=0;j<NUM_ADDREQ;j++)
	{
	  ran->addreq |= averageResults(&results[i].addreq[j],&final_results->addreq[j]);
	}
      for(j=0;j<NUM_LFSR;j++)
	{
	  ran->lfsr |= averageResults(&results[i].lfsr[j],&final_results->lfsr[j]);
	}
      for(j=0;j<NUM_REV_ADDR;j++)
	{
	  ran->rev_addr |= averageResults(&results[i].rev_addr[j],&final_results->rev_addr[j]);
	}
      for(j=0;j<NUM_WALK;j++)
	{
	  ran->walk |= averageResults(&results[i].walk[j],&final_results->walk[j]);
	}
      for(j=0;j<NUM_BANK;j++)
	{
	  ran->bank |= averageResults(&results[i].bank[j],&final_results->bank[j]);
	}
      for(j=0;j<MAX_SIZES;j++)
	{
	  ran->mem_bw |= averageResults(&results[i].mem_bandwidth[0][j],&final_results->mem_bandwidth[0][j]);
	  ran->mem_bw |= averageResults(&results[i].mem_bandwidth[1][j],&final_results->mem_bandwidth[1][j]);
	}

      ran->swap |= averageResults(&results[i].swap[0],&final_results->swap[0]);

      for(j=0;j<NUM_FP_TESTS;j++)
	{
	  for(k=0;k<MAX_SIZES;k++)
	    {
	      ran->int16 |= averageResults(&results[i].int16_rslt[j][k],&final_results->int16_rslt[j][k]);
	      ran->int32 |= averageResults(&results[i].int32_rslt[j][k],&final_results->int32_rslt[j][k]);
	      ran->int64 |= averageResults(&results[i].int64_rslt[j][k],&final_results->int64_rslt[j][k]);
	      ran->sgl |= averageResults(&results[i].sgl_rslt[j][k],&final_results->sgl_rslt[j][k]);
	      ran->dbl |= averageResults(&results[i].dbl_rslt[j][k],&final_results->dbl_rslt[j][k]);
	    }
	}

      for(j=0;j<NUM_DISK_MODES;j++)
	{
	  for(k=0;k<MAX_FILE_SIZES;k++)
	    {
	      for(m=0;m<MAX_DIRS;m++)
		{
		  for(l=0;l<2;l++)
		    {
		      ran->disk |= averageResults(&results[i].disk_rslt[l][j][k][m],&final_results->disk_rslt[l][j][k][m]);
		    }
		}
	    }
	}
    }
}

Boolean averageResults(result *results,result *final)
{
  Boolean ran=FALSE;

  if(results->runs != 0)
    {
      ran = TRUE;
      if(final->runs == 0)
	{
	  strncpy(final->name,results->name,63);
	  final->runs = results->runs;
	  final->errors = results->errors;
	  final->size = results->size;
	  final->total = results->total;
	  final->total_time = results->total_time;
	  final->min = results->min;
	  final->max = results->max;
	}
      else
	{
	  if(final->name[0] == 0) strncpy(final->name,final->name,63);
	  final->runs += results->runs;
	  final->errors += results->errors;
	  final->size = results->size;
	  final->total += results->total;
	  final->total_time += results->total_time;
	  if(final->min > results->min) final->min = results->min;
	  if(final->max < results->max) final->max = results->max;
	}
    }
  results->name[63] = 0;
  return(ran);
}

void clrResults(result_summary *result)
{
  SInt32 j,k,l,m;

  for(j=0;j<NUM_FIXED;j++)
    {
      result->fixed[j].runs = 0;
      result->fixed[j].errors = 0;
      result->fixed[j].min = 0;
      result->fixed[j].max = 0;
      result->fixed[j].total = 0;
      result->fixed[j].total_time = 0;
      result->fixed[j].name[0] = 0;
    }

  for(j=0;j<NUM_ADDREQ;j++)
    {
      result->addreq[j].runs = 0;
      result->addreq[j].errors = 0;
      result->addreq[j].min = 0;
      result->addreq[j].max = 0;
      result->addreq[j].total = 0;
      result->addreq[j].total_time = 0;
      result->addreq[j].name[0] = 0;
	  

    }

  for(j=0;j<NUM_LFSR;j++)
    {
      result->lfsr[j].runs = 0;
      result->lfsr[j].errors = 0;
      result->lfsr[j].min = 0;
      result->lfsr[j].max = 0;
      result->lfsr[j].total = 0;
      result->lfsr[j].total_time = 0;
      result->lfsr[j].name[0] = 0;
	  

    }
  for(j=0;j<NUM_REV_ADDR;j++)
    {
      result->rev_addr[j].runs = 0;
      result->rev_addr[j].errors = 0;
      result->rev_addr[j].min = 0;
      result->rev_addr[j].max = 0;
      result->rev_addr[j].total = 0;
      result->rev_addr[j].total_time = 0;
      result->rev_addr[j].name[0] = 0;
	  

    }
  for(j=0;j<NUM_WALK;j++)
    {
      result->walk[j].runs = 0;
      result->walk[j].errors = 0;
      result->walk[j].min = 0;
      result->walk[j].max = 0;
      result->walk[j].total = 0;
      result->walk[j].total_time = 0;
      result->walk[j].name[0] = 0;
	  

    }

  for(j=0;j<NUM_BANK;j++)
    {
      result->bank[j].runs = 0;
      result->bank[j].errors = 0;
      result->bank[j].min = 0;
      result->bank[j].max = 0;
      result->bank[j].total = 0;
      result->bank[j].total_time = 0;
      result->bank[j].name[0] = 0;
	  

    }

  for(j=0;j<MAX_SIZES;j++)
    {
      for(k=0;k<2;k++)
	{
	  result->mem_bandwidth[k][j].runs = 0;
	  result->mem_bandwidth[k][j].errors = 0;
	  result->mem_bandwidth[k][j].min = 0;
	  result->mem_bandwidth[k][j].max = 0;
	  result->mem_bandwidth[k][j].total = 0;
	  result->mem_bandwidth[k][j].total_time = 0;
	  result->mem_bandwidth[k][j].name[0] = 0;
	} 

    }

  result->swap[0].runs = 0;
  result->swap[0].errors = 0;
  result->swap[0].min = 0;
  result->swap[0].max = 0;
  result->swap[0].total = 0;
  result->swap[0].total_time = 0;
  result->swap[0].name[0] = 0;

  for(j=0;j<NUM_FP_TESTS;j++)
    {
      for(k=0;k<MAX_SIZES;k++)
	{
	  result->sgl_rslt[j][k].runs = 0;
	  result->sgl_rslt[j][k].errors = 0;
	  result->sgl_rslt[j][k].min = 0;
	  result->sgl_rslt[j][k].max = 0;
	  result->sgl_rslt[j][k].total = 0;
	  result->sgl_rslt[j][k].total_time = 0;
	  result->sgl_rslt[j][k].name[0] = 0;

	  result->dbl_rslt[j][k].runs = 0;
	  result->dbl_rslt[j][k].errors = 0;
	  result->dbl_rslt[j][k].min = 0;
	  result->dbl_rslt[j][k].max = 0;
	  result->dbl_rslt[j][k].total = 0;
	  result->dbl_rslt[j][k].total_time = 0;
	  result->dbl_rslt[j][k].name[0] = 0;

	  result->int16_rslt[j][k].runs = 0;
	  result->int16_rslt[j][k].errors = 0;
	  result->int16_rslt[j][k].min = 0;
	  result->int16_rslt[j][k].max = 0;
	  result->int16_rslt[j][k].total = 0;
	  result->int16_rslt[j][k].total_time = 0;
	  result->int16_rslt[j][k].name[0] = 0;

	  result->int32_rslt[j][k].runs = 0;
	  result->int32_rslt[j][k].errors = 0;
	  result->int32_rslt[j][k].min = 0;
	  result->int32_rslt[j][k].max = 0;
	  result->int32_rslt[j][k].total = 0;
	  result->int32_rslt[j][k].total_time = 0;
	  result->int32_rslt[j][k].name[0] = 0;

	  result->int64_rslt[j][k].runs = 0;
	  result->int64_rslt[j][k].errors = 0;
	  result->int64_rslt[j][k].min = 0;
	  result->int64_rslt[j][k].max = 0;
	  result->int64_rslt[j][k].total = 0;
	  result->int64_rslt[j][k].total_time = 0;
	  result->int64_rslt[j][k].name[0] = 0;
	  
	}
    }
  for(j=0;j<NUM_DISK_MODES;j++)
    {
      for(k=0;k<MAX_FILE_SIZES;k++)
	{
	  for(l=0;l<MAX_DIRS;l++)
	    {
	      for(m=0;m<2;m++)
		{
		  result->disk_rslt[m][j][k][l].runs = 0;
		  result->disk_rslt[m][j][k][l].errors = 0;
		  result->disk_rslt[m][j][k][l].min = 0;
		  result->disk_rslt[m][j][k][l].max = 0;
		  result->disk_rslt[m][j][k][l].total = 0;
		  result->disk_rslt[m][j][k][l].total_time = 0;
		  result->disk_rslt[m][j][k][l].name[0] = 0;
		}
	    }
	}
    }
}


void writeResults(FILE *fp,FILE *csv,result_summary results[MAX_THREADS],run_summary *ran,file_info *finfo)
{
  SInt32 test;

  for(test=0;test<NUM_TESTS;test++)
    {
      switch(test)
	{
	case FIXED_MEM0:
	case FIXED_MEM1:
	case FIXED_MEM2:
	case FIXED_MEM3:
	case FIXED_MEM4:
	case FIXED_MEM5:
	case FIXED_MEM6:
	case FIXED_MEM7:
	case FIXED_MEM8:
	case FIXED_MEM9:
	case FIXED_MEM10:
	case FIXED_MEM11:
	case FIXED_MEM12:
	case FIXED_MEM13:
	case FIXED_MEM14:
	case FIXED_MEM15:
	  if(ran->fixed)
	    {
	      if(fp)fprintf(fp,"\nFixed Pattern Memory Tests\n");
	      if(csv)fprintf(csv,"\nFixed Pattern Memory Tests\n");
	      displayFixResults(fp,csv,NUM_FIXED,results->fixed);
	    }
	  break;
	case ADDR_EQ0:
	case ADDR_EQ1:
	case ADDR_EQ2:
	case ADDR_EQ3:
	case ADDR_EQ4:
	case ADDR_EQ5:
	case ADDR_EQ6:
	case ADDR_EQ7:
	case ADDR_EQ8:
	case ADDR_EQ9:
	case ADDR_EQ10:
	case ADDR_EQ11:
	case ADDR_EQ12:
	case ADDR_EQ13:
	case ADDR_EQ14:
	case ADDR_EQ15:
	case ADDR_EQ16:
	case ADDR_EQ17:
	case ADDR_EQ18:
	case ADDR_EQ19:
	case ADDR_EQ20:
	case ADDR_EQ21:
	case ADDR_EQ22:
	case ADDR_EQ23:
	case ADDR_EQ24:
	case ADDR_EQ25:
	case ADDR_EQ26:
	case ADDR_EQ27:
	case ADDR_EQ28:
	case ADDR_EQ29:
	case ADDR_EQ30:
	case ADDR_EQ31:
	  if(ran->addreq)
	    {
	      if(fp)fprintf(fp,"\nAddr == Data Memory Tests\n");
	      if(csv)fprintf(csv,"\nAddr == Data Memory Tests\n");
	      displayFixResults(fp,csv,NUM_ADDREQ,results->addreq);
	    }
	  break;
	case REV_ADDR0:
	case REV_ADDR1:
	  if(ran->rev_addr)
	    {
	      if(fp)fprintf(fp,"\nBit Reverse Address Tests\n");
	      if(csv)fprintf(csv,"\nBit Reverse Address Tests\n");
	      displayFixResults(fp,csv,NUM_REV_ADDR,results->rev_addr);
	    }
	  break;
	case LFSR_TST0:
	case LFSR_TST1:
	case LFSR_TST2:
	case LFSR_TST3:
	case LFSR_TST4:
	case LFSR_TST5:
	case LFSR_TST6:
	case LFSR_TST7:
	  if(ran->lfsr)
	    {
	      if(fp)fprintf(fp,"\nRandom Memory Tests\n");
	      if(csv)fprintf(csv,"\nRandom Memory Tests\n");
	      displayFixResults(fp,csv,NUM_LFSR,results->lfsr);
	    }
	  break;
	case WALK_TST0:
	case WALK_TST1:
	case WALK_TST2:
	case WALK_TST3:
	case WALK_TST4:
	case WALK_TST5:
	case WALK_TST6:
	case WALK_TST7:
	  if(ran->walk)
	    {
	      if(fp)fprintf(fp,"\nWalking One/Zero Memory Tests\n");
	      if(csv)fprintf(csv,"\nWalking One/Zero Memory Tests\n");
	      displayFixResults(fp,csv,NUM_WALK,results->walk);
	    }
	  break;


	case BANK_TST2:
	case BANK_TST3:
	case BANK_TST4:
	case BANK_TST5:
	case BANK_TST6:
	case BANK_TST7:
	case BANK_TST8:
	case BANK_TST9:
	case BANK_TST10:
	case BANK_TST11:
	case BANK_TST12:
	case BANK_TST13:
	case BANK_TST14:
	case BANK_TST15:
	  if(ran->bank)
	    {
	      if(fp)fprintf(fp,"\nMultibank Memory Tests\n");
	      if(csv)fprintf(csv,"\nMultibank Memory Tests\n");
	      displayFixResults(fp,csv,NUM_BANK,results->bank);
	    }
	  break;
	case MEM_BAND:
	  if(ran->mem_bw)
	    {
	      if(fp)fprintf(fp,"\nRead Memory Bandwidth\n");
	      if(csv)fprintf(csv,"\nRead Memory Bandwidth\n");
	      displayBWResults(fp,csv,results->mem_bandwidth[0],"GB/Sec");

	      if(fp)fprintf(fp,"\nWrite Memory Bandwidth\n");
	      if(csv)fprintf(csv,"\nWrite Memory Bandwidth\n");
	      displayBWResults(fp,csv,results->mem_bandwidth[1],"GB/Sec");
	    }
	  break;
	case FP_TST:
	  if(ran->sgl)
	    {
	      if(fp)fprintf(fp,"\nSingle Precision Floating Point Tests\n");
	      if(csv)fprintf(csv,"\nSingle Precision Floating Point Tests\n");
	      displayFPResults(fp,csv,results->sgl_rslt,"MFLOPs");
	    }
	  if(ran->dbl)
	    {
	      if(fp)fprintf(fp,"\nDouble Precision Floating Point Tests\n");
	      if(csv)fprintf(csv,"\nDouble Precision Floating Point Tests\n");
	      displayFPResults(fp,csv,results->dbl_rslt,"MFLOPs");
	    }
	  break;
	case INT_TST:
	  if(ran->int16)
	    {
	      if(fp)fprintf(fp,"\n16 Bit Integer Tests\n");
	      if(csv)fprintf(csv,"\n16 Bit Integer Tests\n");
	      displayFPResults(fp,csv,results->int16_rslt,"MOPs");
	    }
	  if(ran->int32)
	    {
	      if(fp)fprintf(fp,"\n32 Bit Integer Tests\n");
	      if(csv)fprintf(csv,"\n32 Bit Integer Tests\n");
	      displayFPResults(fp,csv,results->int32_rslt,"MOPs");
	    }
	  if(ran->int64)
	    {
	      if(fp)fprintf(fp,"\n64 Bit Integer Tests\n");
	      if(csv)fprintf(csv,"\n64 Bit Integer Tests\n");
	      displayFPResults(fp,csv,results->int64_rslt,"MOPs");
	    }
	  break;
	case DISK_TST:
	  if(ran->disk)
	    {
	      if(fp)fprintf(fp,"\nDisk Tests, Read Bandwidth\n");
	      if(csv)fprintf(csv,"\nDisk Tests, Read Bandwidth\n");
	      displayDiskResults(fp,csv,results->disk_rslt[0],"MB/Sec",finfo);

	      if(fp)fprintf(fp,"\nDisk Tests, Write Bandwidth\n");
	      if(csv)fprintf(csv,"\nDisk Tests, Write Bandwidth\n");
	      displayDiskResults(fp,csv,results->disk_rslt[1],"MB/Sec",finfo);
	    }
	  break;
	case SWAP_TST:
	  if(ran->swap)
	    {
	      if(fp)fprintf(fp,"\nSwap Space Test\n");
	      if(csv)fprintf(csv,"\nSwap Space Test\n");
	      displayFixResults(fp,csv,1L,&results->swap[0]);
	    }
	  break;
		 
	}
    }
}


void displayFPResults(FILE *fp,FILE *csv,result rslt[NUM_FP_TESTS][MAX_SIZES],char *unit)
{
  SInt32 i,j;
  if(fp)fprintf(fp,"\nTest\t\t\tStatus\tRuns\tErrors\tSize\tMin BW\tMax BW\tAvg BW\tTest Time\n");
  if(fp)fprintf(fp,"\t\t\t\t\t\tKWords\t%s\t%s\t%s\tmSecs\n",unit,unit,unit);
  if(csv)fprintf(csv,"\nTest,Status,Runs,Errors,Size,Min BW,Max BW,Avg BW,Test Time\n");
  if(csv)fprintf(csv,",,,,KWords,%s,%s,%s,mSecs\n",unit,unit,unit);
  for(i=0;i<NUM_FP_TESTS;i++)
    {
      for(j=0;j<MAX_SIZES;j++)
	{
	  displayResults(fp,csv,&rslt[i][j]);
	}
    }
}

void displayDiskResults(FILE *fp,FILE *csv,result rslt[MAX_DIRS][NUM_DISK_MODES][MAX_FILE_SIZES],char *unit,file_info *finfo)
{
  SInt32 i,j,k;
  for(i=0;i<MAX_DIRS;i++)
    {
      if(finfo->path[i][0] != 0)
	{
	  if(fp)fprintf(fp,"\nDirectory: %s\n",finfo->path[i]);
	  if(csv)fprintf(csv,"\nDirectory: %s\n",finfo->path[i]);
	  if(fp)fprintf(fp,"Test\t\t\tStatus\tRuns\tErrors\tSize\tMin BW\tMax BW\tAvg BW\tTest Time\n");
	  if(fp)fprintf(fp,"\t\t\t\t\t\tKWords\t%s\t%s\t%s\tmSecs\n",unit,unit,unit);
	  if(csv)fprintf(csv,"Test,Status,Runs,Errors,Size,Min BW,Max BW,Avg BW,Test Time\n");
	  if(csv)fprintf(csv,",,,,KWords,%s,%s,%s,mSecs\n",unit,unit,unit);
	  for(k=0;k<NUM_DISK_MODES;k++)
	    {
	      for(j=0;j<MAX_FILE_SIZES;j++)
		{
		  displayResults(fp,csv,&rslt[i][k][j]);
		}
	    }
	}
    }
}

void displayFixResults(FILE *fp,FILE *csv,SInt32 num,result *rslt)
{
  SInt32 i;
  if(fp)fprintf(fp,"\nTest\t\t\tStatus\tSize\tRuns\tErrors\n");
  //  if(fp)fprintf(fp,"\t\t\t\tMBytes\n");
  if(csv)fprintf(csv,"\nTest,Status,Size,Runs,Errors\n");
  if(csv)fprintf(csv,",,MBytes\n");

  if(num == 1)
    {
      displayMemResults(fp,csv,rslt);
    }
  else
    {
      for(i=0;i<num;i++)
	{
	  displayMemResults(fp,csv,&rslt[i]);
	}
    }
}

void displayBWResults(FILE *fp,FILE *csv,result *rslt,char *unit)
{
  SInt32 i;
  if(fp)fprintf(fp,"\nTest\t\t\tStatus\tRuns\tErrors\tSize\tMin BW\tMax BW\tAvg BW\tTest Time\n");
  if(fp)fprintf(fp,"\t\t\t\t\t\tKWords\t%s\t%s\t%s\tmSecs\n",unit,unit,unit);
  if(csv)fprintf(csv,"\nTest,Status,Runs,Errors,Size,Min BW,Max BW,Avg BW,Test Time\n");
  if(csv)fprintf(csv,",,,,KWords,%s,%s,%s,mSecs\n",unit,unit,unit);
  for(i=0;i<MAX_SIZES;i++)
    {
      displayResults(fp,csv,&rslt[i]);
    }
}

void displayResults(FILE *fp,FILE *csv,result *rslt)
{
  char string[1024],*pass_fail;
  static char *pf[]={"PASS","FAIL"};
  SInt64 size;
  double time;

  if(rslt->runs == 0)
    {
      return;
    }
  else
    {
      if(rslt->errors == 0)
	{
	  pass_fail = pf[0];
	}
      else
	{
	  pass_fail = pf[1];
	}
      size = rslt->size;
      size /= 1024L;
      time = rslt->total_time;
      time /= 1000.0;
      sprintf(string,"%s\t%s\t%lld\t%lld\t%lld\t%0.2f\t%0.2f\t%0.2f\t%0.2f\n",
	      rslt->name,
	      pass_fail,
	      rslt->runs,
	      rslt->errors,
	      size,
	      rslt->min,
	      rslt->max,
	      rslt->total/rslt->total_time,
	      time
	      );
      if(fp)fprintf(fp,"%s",string);

      sprintf(string,"%s,%s,%lld,%lld,%lld,%0.2f,%0.2f,%0.2f,%0.2f\n",
	      rslt->name,
	      pass_fail,
	      rslt->runs,
	      rslt->errors,
	      size,
	      rslt->min,
	      rslt->max,
	      rslt->total/rslt->total_time,
	      time
	      );
      if(csv)fprintf(csv,"%s",string);

    }
}


void displayMemResults(FILE *fp,FILE *csv,result *rslt)
{
  char string[1024],*pass_fail,scale[2];
  static char *pf[]={"PASS","FAIL"};
  double size;
  double time;

  if(rslt->runs == 0)
    {
      return;
    }
  else
    {
      if(rslt->errors == 0)
	{
	  pass_fail = pf[0];
	}
      else
	{
	  pass_fail = pf[1];
	}
      scale[1] = 0;
      time = rslt->total_time;
      time /= 1000.0;

      size = rslt->size;
      if(rslt->size > 0x40000000LL)
	{
	  size /= 0x40000000LL;
	  scale[0] = 'G';
	}
      else if(rslt->size > 0x100000LL)
	{
	  size /= 0x100000LL;
	  scale[0] = 'M';
	}
      else
	{
	  size /= 0x400LL;
	  scale[0] = 'K';
	}

      sprintf(string,"%s\t%s\t%0.2f%s\t%lld\t%lld\n",
	      rslt->name,
	      pass_fail,
	      size,scale,
	      rslt->runs,
	      rslt->errors
	      );
      if(fp)fprintf(fp,"%s",string);

      sprintf(string,"%s,%s,%lld,%lld,%lld\n",
	      rslt->name,
	      pass_fail,
	      rslt->size,
	      rslt->runs,
	      rslt->errors
	      );
      if(csv)fprintf(csv,"%s",string);
    }
}

void commaNumber(double value,char *num)
{
  char *sindx,*dindx,string[256],myChar;
  SInt32 i,j,k,l,remainder;

  sprintf(string,"%0.0f",value);
  string[63] = 0;

  i = 0;
  myChar = string[i];
  if(myChar == 0)
    {
      num[0] = 0;
      return;
    }
  while((myChar != '.') && (myChar != 0))
    {
      i++;
      myChar = string[i];
    }
  dindx = num;
  sindx = string;
  j = (i + 1)/3;
  j *= 3;
  remainder = (i + 1) - j;
  l = 0;
  while(l < i)
    {
      for(k=remainder;k<3;k++)
	{
	  *dindx++ = *sindx++;
	  l++;
	}
      if(l < i) *dindx++ = ',';
      remainder = 0;
    }
  myChar = *sindx++;
  while(myChar != 0)
    {
      *dindx++ = myChar;
      myChar = *sindx++;
      
    }
  *dindx++ = 0;
  
}
long displaySInt16PassFail(char *prefix,SInt16 values[],SInt16 exp[],long count)
{
  char string[1024];
  static char *pass = "PASS         ";
  static char *fail = "FAIL         ";
  long i,errors=0;

  leftJustifyString(prefix,string,16);
  for(i=0;i<count;i++)
    {
      if(values[i] == exp[i])
	{
	  sprintf(string,"%s%s",string,pass);
	}
      else
	{
	  sprintf(string,"%s%s",string,fail);
	  errors++;
	}
    }
  sprintf(string,"%s\n",string);
  printTranscript(string,FALSE);
  return(errors);
  
}

void minMaxTemps(long num_values,double feature_value[],double min[],double max[],Boolean firstTime)
{
  SInt32 i;
  for(i=0;i<num_values;i++)
    {
      if(firstTime)
	{
	  min[i] = feature_value[i];
	  max[i] = feature_value[i];
	}
      else
	{
	  if(min[i] > feature_value[i]) min[i] = feature_value[i];
	  if(max[i] < feature_value[i]) max[i] = feature_value[i];
	}
    }
}

void printRamSize(FILE *fp,FILE *csv,char *name,SInt64 size)
{
  double mb;
  mb = size;
  mb /= 1048576.0;
  if(fp)fprintf(fp,"%s: %0.2f MBytes\n",name,mb);
  if(csv)fprintf(csv,"%s,%0.2f,MBytes\n",name,mb);
  
}

//-- Find a command line arg, must start with -, extra -s are removed
Boolean getSwitch(char *arg,char *string)
{
  char *sindx,*dindx,myChar;
  SInt32 cnt=0;

  string[0] = 0;

  if(arg == NULL) return(FALSE);

  sindx = arg;
  myChar = *sindx++;

  if(myChar == 0) return(FALSE);
  if(myChar != '-') return(FALSE);

  dindx = string;

  while(myChar == '-')
    {
      myChar = *sindx++;
    }

  while((myChar) && (cnt < 64))
    {
      *dindx++ = myChar;
      myChar = *sindx++;
      cnt++;
    }
  *dindx++ = 0;
  return(TRUE);
}

//-- Find a command line switch second arg, must not start with -, extra -s are removed
Boolean getSwitchArg(char *arg,char *string)
{
  char *sindx,*dindx,myChar;
  SInt32 cnt=0;

  string[0] = 0;

  if(arg == NULL) return(FALSE);

  sindx = arg;
  myChar = *sindx++;

  if(myChar == 0) return(FALSE);
  if(myChar == '-') return(FALSE);

  dindx = string;

  while((myChar) && (cnt < 64))
    {
      *dindx++ = myChar;
      myChar = *sindx++;
      cnt++;
    }
  *dindx++ = 0;
  return(TRUE);
}

void clrExpected(exp_data *exp)
{
  SInt64 i,j;
  for(i=0;i<MAX_SIZES;i++)
    {
      for(j=0;j<NUM_FP_TESTS;j++)
	{
	  exp->int16[i][j] = 0;
	  exp->int32[i][j] = 0;
	  exp->int64[i][j] = 0;
	  exp->fp[i][j] = 0;
	  exp->dbl[i][j] = 0;
	  exp->time_scale16[i][j] = 0;
	  exp->time_scale32[i][j] = 0;
	  exp->time_scale64[i][j] = 0;
	  exp->time_scale_fp[i][j] = 0;
	  exp->time_scale_dbl[i][j] = 0;
	}
    }
}

void pad2length(char *string,SInt32 length)
{
  SInt32 i=0;
  char *indx=string,myChar=0;
  myChar = *indx++;
  while(myChar != 0)
    {
      myChar = *indx++;
      i++;
    }
  indx--;
  *indx++ = ' ';
  i++;
  while(i < length)
    {
      *indx++ = ' ';
      i++;
    }
  *indx++ = 0;
}
void getVersion(void)
{
  printf("sys_basher version %s\n",getRevision());
}
void explainIt(void)
{
  printf("sys_basher (c)2006, 2007, 2008, 2009 Polybus Systems Corp,  Revision %s\n",getRevision());
  printf("   Sys_basher is a multithreaded hardware exerciser. Sys_basher tests CPU, memory and disk I/O reliability.\n");
  printf("   It also measures low level system performance.\n");
  printf("Usage: sys_basher [OPTIONS]\n");
  printf("options:\n");
  printf("  -d,--disk                   Run disk tests\n");
  printf("  -du,--dutycyle              Dutycycle n Duty cycle in percent, the default is 100\n");
  printf("                              Inserts Sleeps between tests to reduce the overall CPU duty cycle\n");
  printf("  -f,--fast                   Skip the memory tests, just run the disk, fp and memory bandwidth tests.\n");
  printf("  -fp,--fp                    Run floating point tests\n");
  printf("  -h,--help                   Print this help message\n");
  printf("  -ho #,--hours #             Run for at least # hours.\n");
  printf("  -i,--int                    Run integer tests\n");
  printf("  -l #,--loops #              Number of iterations.\n");
  printf("  -mb,--mbandwidth            Run memory bandwidth\n");
  printf("  -m,--mem [abprwv]           Run memory tests\n");
  printf("     No argument                Run all memory tests\n");
  printf("     a                          Address = Data tests\n");
  printf("     b                          Bank tests\n");
  printf("     p                          Fixed pattern tests\n");
  printf("     r                          Random data tests\n");
  printf("     w                          Walking Ones/Zeros data tests\n");
  printf("     v                          Bit reversed address tests\n");
  printf("  -mi,--minutes #             Run for at least # minutes.\n");
  printf("  -p n0..nx,--path n0..nx     Directory path(s) for the disk tests, upto 32 paths maybe specified. The default path is ./\n");
  printf("  -r name,--report name       Report file name, generate name.rpt and name_xx.csv. Default is sys_basher.rpt and sys_basher_xx.csv\n");
  printf("  -s,--swap                   Force use of swap space. This option is incredibly slow.\n");
  printf("  -st,--stop                  Stop on Error, Exits if error found.\n");
  printf("  -t,--totalmem #             Allocate total ram - (n * 1M) for testing. If -totalmem is not called then the free space is used.\n");
  printf("  -th #,--threads #           Maximum number of threads. Spawn 1-n threads, the default is the number of CPUs.\n");
  printf("  -ts #,--tstart #            Starting thread count, after the initial calibration pass loop from tstart to threads, the default is 1\n");
  printf("  -v,--verbose                Verbose, print detailed results, default is PASS/FAIL only.\n");
  printf("  -ve,--version               Version\n");
  printf("The default is to run all tests unless at least one individual test group is selected.\n");
  printf("Report and log files:\n");
  printf(" The default report_name is sys_basher.\n");
  printf(" The the default name can be overridden with the -r switch.\n");
  printf("  report_name.log             Running log. Syncs are executed after each test so that this file can be preserved up to the point of a crash.\n" );
  printf("  report_name.rpt             Summary report file.\n");
  printf("  report_name_#_threads.csv   Comma separated report files for each thread count. These files can be read into a spreadsheet.\n");

}

char *getRevision(void)
{
  return("1.2.4");
}

